##############################################################################
# Copyright (c) 2004-2005 Black Rock City, LLC
# Copyright (c) 2004-2005 ObjectRealms, LLC
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
##############################################################################
"""
$Id: relations.py 21796 2006-04-04 22:59:53Z rafrombrc $
"""

from Products.Archetypes.ReferenceEngine import Reference

class TeamMembershipRelation(Reference):
    """
    Relationship between a Member and a TeamMembership
    """
    relationship = 'Team Membership Relation'
    def beforeSourceDeleteInformTarget(self):
        """
        When a member is deleted we want to delete their memberships
        as well
        """
        tObj = self.getTargetObject()
        parent = tObj.aq_inner.aq_parent
        parent._delObject(tObj.id)

    def manage_afterAdd(self, item, container):
        """
        Also construct a MemberTeamRelation
        """
        Reference.manage_afterAdd(self, item, container)
        team = self.getTargetObject().getTeam()
        mem = self.getSourceObject()
        mem.addReference(team,
                         relationship=MemberTeamRelation.relationship,
                         referenceClass=MemberTeamRelation)

    def manage_beforeDelete(self, item, container):
        """
        Also tear down the MemberTeamRelation
        """
        Reference.manage_beforeDelete(self, item, container)
        team = self.getTargetObject().getTeam()
        mem = self.getSourceObject()
        mem.deleteReference(team,
                            relationship=MemberTeamRelation.relationship)

class MemberTeamRelation(Reference):
    """
    Relationship between a Member and a Team
    """
    relationship = 'Member Team Relation'

class TeamSpaceTeamRelation(Reference):
    """
    Relationship between a Team and a TeamSpace
    """
    relationship = 'TeamSpace Team Relation'

class TeamRosterTeamRelation(Reference):
    """
    Relationship between a Team and a TeamRoster
    """
    relationship = 'TeamRoster Team Relation'

class TeamContentTeamRelation(Reference):
    """
    Relationship between a Team and a piece of content that is subject
    to that team's security policy
    """
    relationship = 'Team Content Team Relation'
