##############################################################################
# Copyright (c) 2004-2005 Black Rock City, LLC
# Copyright (c) 2004-2005 ObjectRealms, LLC
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
##############################################################################
"""
Description:
 Security Mixin That Modifies Local Roles Structure
 Based on Team Membership.
 
Author: kapil thangavelu <k_vertigo@objectrealms.net>
Date: 4/5/2004
"""

from ExtensionClass import Base
from ComputedAttribute import ComputedAttribute
from relations import TeamContentTeamRelation

DEBUG = 0

class TeamSecurity(Base):
    """
    Security Policy Enforcement Taking into Account Team Membership
    """

    # subclasses return which teams whose membership will grant
    # additional local roles they are interested in.
    def _getTeamsForLocalRoles(self):
        raise NotImplemented

    def _getTeamLocalRoles(self):
        """
        include members of this team with their assigned team roles
        as local roles of the workspace.

        XXX cache me
        """
        res = {}
        for t in self._getTeamsForLocalRoles():
            for m in t.getActiveMemberships():
                roles = res.setdefault( m.id, [] )
                mroles = m.getSecurityRoles()
                [ roles.append(mr) for mr in mroles if not mr in roles ]
        if self.__ac_local_roles_bag__:
            bag = self.__ac_local_roles_bag__
            for mid in bag:
                roles = res.setdefault( mid, [] )
                broles = bag[mid]
                [ roles.append(br) for br in broles if not br in roles ]
        if res:
            return res
        else:
            return None

    def _getLocalRolesDebugShim( self ):
        try:
            return self._getTeamLocalRoles()
        except:
            import pdb, sys
            ec, e, tb = sys.exc_info()
            print "Error Getting Local Roles", ec, e
            if DEBUG:
                pdb.post_mortem( tb )
            else:
                raise

    __ac_local_roles__ = ComputedAttribute(_getTeamLocalRoles, 1)
    __ac_local_roles_bag__ = None

    #################################
    # rewrite local roles support to use bag attribute as we
    # are overriding the base variable, this bag is included
    # in the computed local roles and supplements team assignments

    def has_local_roles(self):
        dict=self.__ac_local_roles__ or {}
        if not len(dict):
            return len(self.__ac_local_roles__)
        return len(dict)

    def get_local_roles(self):
        dict=self.__ac_local_roles__ or {}
        keys=dict.keys()
        keys.sort()
        info=[]
        for key in keys:
            value=tuple(dict[key])
            info.append((key, value))
        return tuple(info)

    def get_local_roles_for_userid(self, userid):
        dict=self.__ac_local_roles__ or {}
        return tuple(dict.get(userid, []))
    
    def manage_addLocalRoles(self, userid, roles, REQUEST=None):
        """Set local roles for a user."""
        self._v_role_cache = None
        if not roles:
            raise ValueError, 'One or more roles must be given!'
        dict=self.__ac_local_roles_bag__ or {}
        local_roles = list(dict.get(userid, []))
        for r in roles:
            if r not in local_roles:
                local_roles.append(r)
        dict[userid] = local_roles
        self.__ac_local_roles_bag__=dict
        if REQUEST is not None:
            stat='Your changes have been saved.'
            return self.manage_listLocalRoles(self, REQUEST, stat=stat)

    def manage_setLocalRoles(self, userid, roles, REQUEST=None):
        """Set local roles for a user."""
        if not roles:
            raise ValueError, 'One or more roles must be given!'
        dict=self.__ac_local_roles_bag__ or {}
        dict[userid]=roles
        self.__ac_local_roles_bag__=dict
        if REQUEST is not None:
            stat='Your changes have been saved.'
            return self.manage_listLocalRoles(self, REQUEST, stat=stat)

    def manage_delLocalRoles(self, userids, REQUEST=None):
        """Remove all local roles for a user."""
        dict=self.__ac_local_roles_bag__ or {}
        for userid in userids:
            if dict.has_key(userid):
                del dict[userid]
        self.__ac_local_roles_bag__=dict
        if REQUEST is not None:
            stat='Your changes have been saved.'
            return self.manage_listLocalRoles(self, REQUEST, stat=stat)


class TeamSecurityMixin(TeamSecurity):
    """ this can be used as a mix-in for any AT content type to allow
        that type to honor the security policy of any teams that are
        referenced with the TeamContentTeamRelation reference class"""

    def _getTeamsForLocalRoles(self):
        """ retrieves all teams for which this content implements the
            security policy """
        return self.getRefs(relationship = TeamContentTeamRelation.relationship)
                
